<?php

class Database {

    private $_db = null;

    /**
     * Constructor, makes a database connection
     */
    public function __construct() {

        try {
            $this->_db = new PDO('mysql:host='.DB_HOST.';dbname='.DB_NAME, DB_USER, DB_PASS, array( 
      			PDO::MYSQL_ATTR_USE_BUFFERED_QUERY => true
   			));
            $this->_db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
            $this->_db->query('SET CHARACTER SET utf8');
        } catch (PDOException $e) {
            exit('Error while connecting to database.'.$e->getMessage());
        }
    }

    private function printErrorMessage($message) {
        echo $message;
    }

    /**
	 * Get Users
	 */
	 public function getUsers() {
        try {
            $sth = $this->_db->prepare("SELECT id FROM users");
			$sth->setFetchMode(PDO::FETCH_ASSOC);
            $sth->execute();
            $rows = $sth->fetchAll(PDO::FETCH_ASSOC);
            return $rows;
        } catch (PDOException $e) {
            $this->printErrorMessage($e->getMessage());
        }
	 }
	 
    /**
     * Get login 
     */
     public function getLogin($username, $password) {
        try {
            $sth = $this->_db->prepare("SELECT id FROM users WHERE username= ? AND password= ? ");
            $sth->bindValue(1, $username, PDO::PARAM_STR);
            $sth->bindValue(2, $password, PDO::PARAM_STR);
            $sth->execute();
            $row = $sth->fetch(PDO::FETCH_OBJ);
			return $row->id;
        } catch (PDOException $e) {
            $this->printErrorMessage($e->getMessage());
        }
    }

    /**
     * Get Suppliers 
     */
     public function getSuppliers() {
        try {
            $sth = $this->_db->prepare("SELECT id,name FROM energy_supplier ORDER BY name ASC");
            $sth->setFetchMode(PDO::FETCH_ASSOC);
            $sth->execute();
            $rows = $sth->fetchAll(PDO::FETCH_ASSOC);
            return $rows;
        } catch (PDOException $e) {
            $this->printErrorMessage($e->getMessage());
        }
    }

	/**
     * Get SuppliersPackages 
     */
     public function getSuppliersPackages($supplier_id) {
        try {
            $sth = $this->_db->prepare("SELECT id,package_name FROM energy_packages WHERE supplier_id= ? ORDER BY package_name ASC");
            $sth->bindValue(1, $supplier_id, PDO::PARAM_STR);
            $sth->setFetchMode(PDO::FETCH_ASSOC);
            $sth->execute();
            $rows = $sth->fetchAll(PDO::FETCH_ASSOC);
            return $rows;
        } catch (PDOException $e) {
            $this->printErrorMessage($e->getMessage());
        }
    }

    /**
     * Update login 
     */
     public function updateLogin($password, $user_id) {
        try {
            $sth = $this->_db->prepare("UPDATE users SET password= ? WHERE id= ?");
            $sth->bindValue(1, $password, PDO::PARAM_STR);
	    $sth->bindValue(2, $user_id, PDO::PARAM_STR);			
            $sth->execute();
			return $sth->rowCount();
        } catch (PDOException $e) {
            $this->printErrorMessage($e->getMessage());
        }
    }
    
    /**
     * Update username 
     */
     public function updateUsername($username, $user_id) {
        try {
            $sth = $this->_db->prepare("UPDATE users SET username= ? WHERE id= ?");
            $sth->bindValue(1, $username, PDO::PARAM_STR);
			$sth->bindValue(2, $user_id, PDO::PARAM_STR);			
            $sth->execute();
			return $sth->rowCount();
        } catch (PDOException $e) {
            $this->printErrorMessage($e->getMessage());
        }
    }
	/**
     * Update settings
     */
    public function updateSettings($key, $value, $user_id) {
        try {
			$sth = $this->_db->prepare("SELECT id FROM settings WHERE `user_id`= ? AND `key`= ?");
			$sth->bindValue(1, $user_id, PDO::PARAM_STR);			
			$sth->bindValue(2, $key, PDO::PARAM_STR);			
			$sth->execute();
			if ($sth->rowCount() > 0) {
				$sth = $this->_db->prepare("UPDATE settings SET `value`= ? WHERE `key`= ? AND `user_id`= ?");
				$sth->bindValue(1, $value, PDO::PARAM_STR);
				$sth->bindValue(2, $key, PDO::PARAM_STR);			
				$sth->bindValue(3, $user_id, PDO::PARAM_STR);			
			} else {
				$sth = $this->_db->prepare("INSERT INTO settings (`id`, `value`,`key`, `user_id`) VALUES (NULL, :value, :key, :user_id)");
				$sth->bindValue(':value', $value, PDO::PARAM_STR);
				$sth->bindValue(':key', $key, PDO::PARAM_STR);			
				$sth->bindValue(':user_id', $user_id, PDO::PARAM_STR);			
			}			
            $sth->execute();
            
			return $sth->rowCount();
       } catch (PDOException $e) {
            $this->printErrorMessage($e->getMessage());
        }
    }    
        
	/**
	 * Get YouLess Devices registered to a single account
	 */
	public function getYouLess($user_id) {
		try {
            $sth = $this->_db->prepare("SELECT * FROM youless WHERE user_id= ?");
            $sth->bindValue(1, $user_id, PDO::PARAM_STR);
            $sth->setFetchMode(PDO::FETCH_ASSOC);
            $sth->execute();
            $youless = $sth->fetchAll(PDO::FETCH_ASSOC);
        } catch (PDOException $e) {
            $this->printErrorMessage($e->getMessage());
        }
        return $youless;
    }

	/**
	 * Get YouLess Devices registered to a single account
	 */
	public function getYouLesses() {
		try {
            $sth = $this->_db->prepare("SELECT * FROM youless");
            $sth->setFetchMode(PDO::FETCH_ASSOC);
            $sth->execute();
            $youless = $sth->fetchAll(PDO::FETCH_ASSOC);
        } catch (PDOException $e) {
            $this->printErrorMessage($e->getMessage());
        }
        return $youless;
    }
	
	/**
	 * Save new YouLess Device
	 */
	public function updateYouLess($user_id, $friendlyname, $YL_address, $YL_port, $YL_password) {
		try {
            $sth = $this->_db->prepare("INSERT INTO youless (`youless_id`,`user_id`,`friendly_name`,`YL_address`,`YL_port`,`YL_password`) VALUES (NULL,?,?,?,?,?)");
            $sth->bindValue(1, $user_id, PDO::PARAM_STR);
            $sth->bindValue(2, $friendlyname, PDO::PARAM_STR);
            $sth->bindValue(3, $YL_address, PDO::PARAM_STR);
            $sth->bindValue(4, $YL_port, PDO::PARAM_STR);
            $sth->bindValue(5, $YL_password, PDO::PARAM_STR);
            $sth->execute();
            $youless = $sth->rowCount();
        } catch (PDOException $e) {
            $this->printErrorMessage($e->getMessage());
        }
        return $youless;
    }    
        
    /**
     * Get settings 
     */
     public function getSettings($user_id) {
        try {
            $sth = $this->_db->prepare("SELECT * FROM settings WHERE user_id= ?");
            $sth->bindValue(1, $user_id, PDO::PARAM_STR);
            $sth->setFetchMode(PDO::FETCH_ASSOC);
            $sth->execute();
            $rows = $sth->fetchAll(PDO::FETCH_ASSOC);
            $settings = array();
            foreach($rows as $k => $v)
            {
            	$settings[$v['key']] = $v['value'];
            }
        } catch (PDOException $e) {
            $this->printErrorMessage($e->getMessage());
        }
        try {
            $sth = $this->_db->prepare("SELECT username FROM users WHERE id= ? ");
            $sth->bindValue(1, $user_id, PDO::PARAM_STR);
            $sth->execute();
            $row = $sth->fetch(PDO::FETCH_OBJ);
			$settings['username'] = $row->username;
        } catch (PDOException $e) {
            $this->printErrorMessage($e->getMessage());
        }
            return $settings;
    }    

  
	/**
	* Get specific day
	*/
    public function getSpecificDay($date, $user_id) {
        try {
            $sth = $this->_db->prepare("
            SELECT
            	*
            FROM 
            	data_h
            WHERE
            	DATE_FORMAT(time, '%Y-%m-%d') = ? AND user_id= ?
            ORDER BY  
				youless_id ASC, time ASC");
			$sth->bindValue(1, $date, PDO::PARAM_STR);           			
            $sth->bindValue(2, $user_id, PDO::PARAM_STR);
            $sth->setFetchMode(PDO::FETCH_ASSOC);
            $sth->execute();
            $rows = $sth->fetchAll(PDO::FETCH_OBJ);
			return $rows;
        } catch (PDOException $e) {
            $this->printErrorMessage($e->getMessage());
        }
    }

	/**
	* Get specific range
	*/
    public function getSpecificRange($begin, $end, $user_id) {
        try {
            $sth = $this->_db->prepare("
            SELECT
            	*
            FROM 
            	data_h
            WHERE
            	(DATE_FORMAT(time, '%Y-%m-%d') BETWEEN ? AND ?) AND user_id= ?
            ORDER BY  
				time ASC");

			$sth->bindValue(1, $begin, PDO::PARAM_STR);  
			$sth->bindValue(2, $end, PDO::PARAM_STR);  			         			
            $sth->bindValue(3, $user_id, PDO::PARAM_STR);
            $sth->setFetchMode(PDO::FETCH_ASSOC);
            $sth->execute();
            $rows = $sth->fetchAll(PDO::FETCH_OBJ);
			return $rows;
        } catch (PDOException $e) {
            $this->printErrorMessage($e->getMessage());
        }
    }  
    
	/**
	* Get month
	*/
    public function getMonth($date, $user_id) {
        try {
            $sth = $this->_db->prepare("
            SELECT
            	time,
            	GROUP_CONCAT(value) as value
            FROM 
            	data_h
            WHERE
            	DATE_FORMAT(time, '%Y-%m') = ? AND user_id= ?
            GROUP BY
            	DATE_FORMAT(time, '%d')
            ORDER BY  
				time ASC");
			$sth->bindValue(1, $date, PDO::PARAM_STR);   			         			
            $sth->bindValue(2, $user_id, PDO::PARAM_STR);
            $sth->setFetchMode(PDO::FETCH_ASSOC);
            $sth->execute();
            $rows = $sth->fetchAll(PDO::FETCH_OBJ);
			return $rows;
        } catch (PDOException $e) {
            $this->printErrorMessage($e->getMessage());
        }
    }    
    
	/**
	* Get kwh count
	*/
    public function getKwhCount($datetime, $user_id) {
        try {
            $sth = $this->_db->prepare("
			SELECT 
				*,
				if((SIGN(timestampdiff(second, inserted,:date)) = -1),
				( ( (timestampdiff(second, inserted,:date))*(timestampdiff(second, inserted,:date)) ) / (-1*(timestampdiff(second, inserted,:date))) ),
				(timestampdiff(second, inserted,:date)) ) as TimeDif
			FROM 
				kwh_h
			WHERE 
				(inserted <= :date OR inserted > :date) AND user_id= :userid
			ORDER BY 
				TimeDif ASC 
			LIMIT 1;");
			$sth->bindValue(':date', $datetime, PDO::PARAM_STR);           			
            $sth->bindValue(':userid', $user_id, PDO::PARAM_STR);
            $sth->setFetchMode(PDO::FETCH_ASSOC);
            $sth->execute();
            $row = $sth->fetch(PDO::FETCH_OBJ);
			return $row;
        } catch (PDOException $e) {
            $this->printErrorMessage($e->getMessage());
        }
    }     
	
   /**
    * Add hourly data (cronjob)
    */ 
    public function addHourlyData($time, $unit, $delta, $values, $user_id, $youless_id) {
        try {
            $sth = $this->_db->prepare("INSERT INTO data_h (
            	time,
				unit,
				delta,
				value,
				user_id,
				youless_id
            ) VALUES (
            	?,
				?,
				?,
				?,
				?,
				?
            )");
            $sth->bindValue(1, $time, PDO::PARAM_STR);
			$sth->bindValue(2, $unit, PDO::PARAM_STR);
			$sth->bindValue(3, $delta, PDO::PARAM_INT);
			$sth->bindValue(4, $values, PDO::PARAM_STR);
			$sth->bindValue(5, $user_id, PDO::PARAM_STR);
			$sth->bindValue(6, $youless_id, PDO::PARAM_STR);
            $sth->execute();
        } catch (PDOException $e) {
            $this->printErrorMessage($e->getMessage());
        }
    } 	
    
   /**
    * Add hourly kwh (cronjob)
    */ 
    public function addHourlyKwh($kwh, $user_id, $youless_id) {
        try {
            $sth = $this->_db->prepare("INSERT INTO kwh_h (
            	kwh,
				user_id,
				youless_id
            ) VALUES (
				?,
				?,
				?
            )");
            $sth->bindValue(1, $kwh, PDO::PARAM_STR);
			$sth->bindValue(2, $user_id, PDO::PARAM_STR);
			$sth->bindValue(3, $youless_id, PDO::PARAM_STR);
            $sth->execute();
        } catch (PDOException $e) {
            $this->printErrorMessage($e->getMessage());
        }
    }     
}
?>